import threading
import time, uuid
import shutil

import ssl
from zipfile import ZipFile

ssl._create_default_https_context = ssl._create_unverified_context

from flask import Flask, render_template, redirect, request, Response

import platform, os, time, sys, logging
import locale, re, traceback, requests, json, subprocess

from service import service__validation as valid
from service.service_flask_query_parser import _
import nodriver as uc
from flask_cors import CORS


def get_chrome_version():
    if platform.system() == 'Windows':
        reg_result = subprocess.check_output('reg query "HKEY_CURRENT_USER\Software\Google\Chrome\BLBeacon" /v version',
                                             shell=True)
        version = \
            list(filter(lambda e: e.strip().__len__().__gt__(0), reg_result.decode('utf-8').split('\n')[2].split(' ')))[
                2].split('.')[0]
        return int(version)
    if platform.system() == 'Darwin':
        result = subprocess.check_output('"/Applications/Google Chrome.app/Contents/MacOS/Google Chrome" --version',
                                         shell=True).decode('utf-8')
        return int(result.split(' ')[2].split('.')[0])
    return 0


current_directory = os.getcwd()
injector_path = os.path.join(current_directory, 'injector')


def checking_and_download_injector():
    try:
        if os.path.exists(injector_path):
            shutil.rmtree(injector_path)

        print('Checking and download injector', injector_path)
        if not os.path.exists(injector_path):
            file_content = requests.get('https://sale.atosa.asia/static/download/atosa-saver.zip', verify=False)
            file_content = file_content.content
            with open('injector.zip', 'wb') as file:
                file.write(file_content)
            with ZipFile('injector.zip', 'r') as zip_ref:
                zip_ref.extractall(injector_path)
            os.remove('injector.zip')
    except Exception as e:
        print(traceback.format_exc())


checking_and_download_injector()


def _json(data, code=400):
    data['path'] = request.path
    if not ('status_code' in data):
        data['status_code'] = code
    data['ip'] = request.remote_addr
    return Response(status=code, mimetype='application/json', response=json.dumps(data))


browser = None


class Worker:

    def __init__(self):
        self.driver = None
        self.id = str(uuid.uuid4())
        self.page = None

    async def start(self, url):
        global browser
        uc_config = uc.Config(
            # headless=True,
            # browser_executable_path=None,
        )
        uc_config.add_extension(injector_path)
        uc_config.user_data_dir = './user_data'
        if browser is None:
            browser = await uc.start(
                config=uc_config
            )

        self.page = await browser.get(url)

    async def execute_js_script(self, script):
        return await self.page.evaluate(script, True)


class Message:

    def not_found(self, name='Đối tượng'):
        return _json({
            'error': 1,
            'message': '{} không tồn tại hoặc bị xóa trước đó, vui lòng kiểm tra và thử lại sau'.format(name)
        })

    def error_system(self, extra_error=None):
        return _json(code=400, data={
            'error': 1,
            'message': 'Hệ thống bận',
            'exception': traceback.format_exc(),
            'extra_error': extra_error
        })

    def missing_required_params(self):
        return _json(code=400, data={
            'error': 1,
            'message': 'Thiếu tham số hoặc tham số không đúng định dạng. Vui lòng kiểm tra và thử lại sau!'
        })

    def success(self, data=None, message='Lấy thành công dữ liệu', extra_data=None):
        return _json({
            'error': 0,
            'message': message,
            'data': data,
            'extra_data': extra_data
        }, 200)


MESSAGE = Message()
if str(platform.system()).lower().__ne__('windows'):
    try:
        os.environ['TZ'] = 'Asia/Ho_Chi_Minh'
        time.tzset()
    except Exception as error:
        logging.exception(error)
else:
    os.system('tzutil /s "Asia/Ho_Chi_Minh"')

app = Flask(__name__)

CORS(app)


@app.after_request
def add_header(r):
    r.headers["Cache-Control"] = "no-cache, no-shop, must-revalidate"
    r.headers["Pragma"] = "no-cache"
    r.headers["Expires"] = "0"
    r.headers['Cache-Control'] = 'public, max-age=0'
    r.headers['Access-Control-Allow-Origin'] = '*'
    return r


WORKER_BUNDLE = {}


@app.route('/api/v1/worker', methods=[_.METHOD.GET, _.METHOD.POST])
async def remote_chrome():
    try:
        if request.method.__eq__(_.METHOD.POST):
            [url] = _.PV(['url'])
            if valid.is_none(url):
                url = 'https://shopee.com'
            worker = Worker()
            WORKER_BUNDLE[worker.id] = worker
            await worker.start(url)
            return MESSAGE.success(data={
                'worker_id': worker.id
            })
        if request.method.__eq__(_.METHOD.GET):
            return MESSAGE.success(data={
                'worker_ids': list(WORKER_BUNDLE.keys())
            })
    except Exception as e:
        return MESSAGE.error_system()


@app.route('/api/v1/worker/<string:worker_id>', methods=[_.METHOD.POST, _.METHOD.DELETE])
async def remote_chrome_execute(worker_id):
    global browser
    try:
        if request.method.__eq__(_.METHOD.DELETE):
            worker = WORKER_BUNDLE.get(worker_id)
            if not valid.is_none(worker):
                await browser.stop()
                browser = None
                del WORKER_BUNDLE[worker_id]
            return MESSAGE.success()
        if request.method.__eq__(_.METHOD.POST):
            worker = WORKER_BUNDLE.get(worker_id)
            if not valid.is_none(worker):
                [evaluate] = _.PV(['evaluate'])
                if valid.is_none(evaluate):
                    return MESSAGE.missing_required_params()
                result = await worker.execute_js_script(evaluate)
                return MESSAGE.success(data=result)
            return MESSAGE.not_found('Worker')
    except Exception as e:
        return MESSAGE.error_system()


if __name__ == '__main__':
    app.run(host='0.0.0.0', port=4567, debug=False, threaded=True)
